/*

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is jslib team code.
The Initial Developer of the Original Code is jslib team.

Portions created by jslib team are
Copyright (C) 2000 jslib team.  All
Rights Reserved.

Original Author: Pete Collins <pete@mozdev.org>
Contributor(s):
  Henrik Gemal <http://gemal.dk>
*/

/************** DUBUG ******************/
if (typeof(su_JS_LIB_LOADED) == "boolean") 
{

  const su_JS_DEBUG_LOADED     = true;
  const su_JS_DEBUG_FILE       = "debug.js";
  const su_jslibConsoleService = su_jslibGetService("@mozilla.org/consoleservice;1", 
                                              "nsIConsoleService");

	const su_DEBUG_ENABLED = true;
	
  /****************************************************************
  * void su_jslibDebug(aOutString)                                   *
  * aOutString is an argument of string debug message             *
  * returns void                                                  *
  *   eg:                                                         * 
  *       var msg="Testing function";                             *
  *       su_jslibDebug(msg);                                        *
  *                                                               *
  *   outputs: Testing function                                   *
  ****************************************************************/

  function su_jslibDebug(aMsg) 
  {
    if (!su_JS_LIB_DEBUG)
      return; 

    if (su_JS_LIB_DEBUG_ALERT)
      alert(aMsg);

    su_jslibDumpInternal(aMsg+'\n');
  }

  // DEPRECATED 
  function su_jslib_debug(aMsg) { return su_jslibDebug(aMsg); }

	// This is for debugging.  It writes all arguments to a console message.
	function su_dd()
	{
		if (!su_DEBUG_ENABLED)
			return;
			
		var consoleService = false;

		consoleService = Components.classes["@mozilla.org/consoleservice;1"].
					getService(Components.interfaces.nsIConsoleService);

		if (consoleService) {
			var i;
			var str = "";

			for (i = 0; i < arguments.length; i++) {
				str += "\n" + arguments[i];
			}

			consoleService.logStringMessage("StumbleUpon debug message: " + str);
		}
	}

  // print to stdout
  function su_jslibPrint(aMsg) { su_jslibDumpInternal(aMsg+'\n'); }

  // print to stdout
  function su_jslibPrintDebug(aMsg, aOutString) 
  {
    if (!aMsg) aMsg = "JSLIB_DEBUG: ";
    su_jslibDumpInternal(aMsg+" "+aOutString+'\n');
  }

  // same as above
  function su_jslibDebugMsg (aMsg, aOutString) { su_jslibPrintDebug(aMsg, aOutString); }

  // dump w/ bracket
  function su_jslibDebugMsgBracket (aMsg, aOutString) { su_jslibPrintDebug(aMsg, "["+aOutString+"]"); }

  // print w/ bracket
  function su_jslibPrintBracket(aOutString) 
  {
    su_jslibDumpInternal("["+aOutString+']\n');
  }

  // print message
  function su_jslibPrintMsg(aOutStr1, aOutStr2) 
  {
    su_jslibDumpInternal(aOutStr1+": "+aOutStr2+"\n");
  }


  /****************************************************************
  * void su_jslibError(e)                                            *
  * e        - argument of results exception                      *
  * returns e.result                                              *
  *   Ex:                                                         * 
  *       su_jslibError(e)                                           *
  *                                                               *
  *   outputs:                                                    *
  *       -----======[ ERROR ]=====-----                          *
  *       Error in jslib.js: su_include:  Missing file path argument *
  *                                                               *
  *       NS_ERROR_NUMBER:   NS_ERROR_XPC_NOT_ENOUGH_ARGS         *
  *       ------------------------------                          *
  *                                                               *
  ****************************************************************/

  function su_jslibError(e) 
  {
    if (su_jslibTypeIsStr(e))
      return su_jslibErrorMsg(e);

    var rv = null;
    var errMsg="";
    if (su_jslibTypeIsObj(e)) {
      var m, n, r, l, ln, fn = "";
      try {
        rv = -e.result;
        m  = e.message;
        fn = e.filename;
        l  = e.location; 
        ln = l.lineNumber; 
      } catch (e) {}
      errMsg+="FileName:          "+fn+"\n"           +
              "Result:            "+rv+"\n"           +
              "Message:           "+m+"\n"            +
              "LineNumber:        "+ln+"\n";
    }

    errMsg = "\n-----======[ jsLib ERROR ]=====-----\n" + errMsg;
    errMsg += "-------------------------------------\n";

    su_jslibDebug(errMsg);

    return rv;
  }

  function su_jslibErrorWarn (e)
  {
    su_jslibDebug("jsLib warn: "+e);
    return null;
  }

  function su_jslibErrorMsg (e, comment)
  {
    typeof(comment) == "string" ? su_jslibDebugMsgBracket(e, comment) : su_jslibDebug(e);
    return -su_jslibRes[e];
  }

  function su_jslibPrintProperties (aObj)
  {
    for (var list in aObj)
      su_jslibPrint(list);
  }

  function su_jslibGetProperties (aObj)
  {
    var out = new Array ();
    for (var list in aObj)
      out.push(list);

    return out;
  }

  function su_jslibPrintType (aObj)
  {
      su_jslibPrint(typeof(aObj));
  }

  function su_jslibPrintTypeWName (aName, aObj)
  {
      su_jslibPrint("Name: "+aName+" JSType: "+typeof(aObj));
  }

  function su_jslibTypeIsObj (aType)
  {
    return (aType && typeof(aType) == "object");
  }

  function su_jslibTypeIsFunc (aType)
  {
    return (aType && typeof(aType) == "function");
  }

  function su_jslibTypeIsStr (aType)
  {
    return (aType && typeof(aType) == "string");
  }

  function su_jslibTypeIsNum (aType)
  {
    return (aType && typeof(aType) == "number");
  }

  function su_jslibTypeIsUndef (aType)
  {
    return (typeof(aType) == "undefined");
  }

  function su_jslibDumpConsole (aMsg)
  {
    su_jslibConsoleService.logStringMessage(aMsg);
  }

  function su_jslibDumpInternal (aMsg)
  {
    dump(aMsg);
    su_jslibDumpConsole(aMsg);
  }

  // Welcome message
  su_jslibDebug('*** load: '+su_JS_DEBUG_FILE+' OK');
  su_jslibDebug(su_JS_LIB_HELP);
  su_jslibDebug("\n\n*********************\nsu_JS_LIB DEBUG IS ON\n*********************\n\n");

} 
 
                                                                                                    

