/*** -*- Mode: Javascript; tab-width: 2;

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is Collabnet code.
The Initial Developer of the Original Code is Collabnet.

Portions created by Collabnet are Copyright (C) 2000 Collabnet.
All Rights Reserved.

Contributor(s): Pete Collins, Doug Turner, Brendan Eich, Warren Harris,
                Eric Plaster, Martin Kutschker


JS Directory Class API

  dirUtils.js

Function List


Instructions:


*/

if(typeof(su_JS_LIB_LOADED)=='boolean') {

const su_JS_DIRUTILS_FILE              = "dirUtils.js";
const su_JS_DIRUTILS_LOADED            = true;

const su_JS_DIRUTILS_FILE_DIR_CID      = "@mozilla.org/file/directory_service;1";

const su_JS_DIRUTILS_I_PROPS           = "nsIProperties";
const su_JS_DIRUTILS_NSIFILE           = su_jslibI.nsIFile;

const su_NS_APP_PREFS_50_DIR           = "PrefD"; // /root/.mozilla/Default User/k1m30xaf.slt
const su_NS_APP_CHROME_DIR             = "AChrom"; // /usr/src/mozilla/dist/bin/chrome
const su_NS_APP_USER_PROFILES_ROOT_DIR = "DefProfRt";  // /root/.mozilla
const su_NS_APP_USER_PROFILE_50_DIR    = "ProfD";      // /root/.mozilla/Default User/k1m30xaf.slt

const su_NS_APP_APPLICATION_REGISTRY_DIR  = "AppRegD"; // /root/.mozilla
const su_NS_APP_APPLICATION_REGISTRY_FILE = "AppRegF"; // /root/.mozilla/appreg
const su_NS_APP_DEFAULTS_50_DIR           = "DefRt";   // /usr/src/mozilla/dist/bin/defaults 
const su_NS_APP_PREF_DEFAULTS_50_DIR      = "PrfDef";  // /usr/src/mozilla/dist/bin/defaults/pref
const su_NS_APP_PROFILE_DEFAULTS_50_DIR   = "profDef"; // /usr/src/mozilla/dist/bin/defaults/profile/US
const su_NS_APP_PROFILE_DEFAULTS_NLOC_50_DIR = "ProfDefNoLoc"; // /usr/src/mozilla/dist/bin/defaults/profile 
const su_NS_APP_RES_DIR                      = "ARes"; // /usr/src/mozilla/dist/bin/res
const su_NS_APP_PLUGINS_DIR                  = "APlugns"; // /usr/src/mozilla/dist/bin/plugins
const su_NS_APP_SEARCH_DIR                   = "SrchPlugns"; // /usr/src/mozilla/dist/bin/searchplugins
const su_NS_APP_PREFS_50_FILE                = "PrefF"; // /root/.mozilla/Default User/k1m30xaf.slt/prefs.js
const su_NS_APP_USER_CHROME_DIR              = "UChrm"; // /root/.mozilla/Default User/k1m30xaf.slt/chrome
const su_NS_APP_LOCALSTORE_50_FILE           = "LclSt"; // /root/.mozilla/Default User/k1m30xaf.slt/localstore.rdf
const su_NS_APP_HISTORY_50_FILE              = "UHist"; // /root/.mozilla/Default User/k1m30xaf.slt/history.dat
const su_NS_APP_USER_PANELS_50_FILE          = "UPnls"; // /root/.mozilla/Default User/k1m30xaf.slt/panels.rdf
const su_NS_APP_USER_MIMETYPES_50_FILE       = "UMimTyp"; // /root/.mozilla/Default User/k1m30xaf.slt/mimeTypes.rdf
const su_NS_APP_BOOKMARKS_50_FILE            = "BMarks"; // /root/.mozilla/Default User/k1m30xaf.slt/bookmarks.html 
const su_NS_APP_SEARCH_50_FILE               = "SrchF"; // /root/.mozilla/Default User/k1m30xaf.slt/search.rdf
const su_NS_APP_MAIL_50_DIR                  = "MailD"; // /root/.mozilla/Default User/k1m30xaf.slt/Mail
const su_NS_APP_IMAP_MAIL_50_DIR             = "IMapMD"; // /root/.mozilla/Default User/k1m30xaf.slt/ImapMail
const su_NS_APP_NEWS_50_DIR                  = "NewsD"; // /root/.mozilla/Default User/k1m30xaf.slt/News
const su_NS_APP_MESSENGER_FOLDER_CACHE_50_DIR = "MFCaD"; // /root/.mozilla/Default User/k1m30xaf.slt/panacea.dat

// Useful OS System Dirs
const su_NS_OS_CURRENT_PROCESS_DIR = "CurProcD"; // /usr/src/mozilla/dist/bin
const su_NS_OS_HOME_DIR = "Home"; // /root
const su_NS_OS_TEMP_DIR = "TmpD"; // /tmp
const su_NS_XPCOM_COMPONENT_DIR = "ComsD"; // /usr/src/mozilla/dist/bin/components

// constructor
function su_DirUtils() {}

su_DirUtils.prototype = {

getPath   : function (aAppID) 
{
  if(!aAppID)
    return su_jslibErrorMsg("NS_ERROR_INVALID_ARG");

  var rv;
  try { 
    rv = su_jslibGetService(su_JS_DIRUTILS_FILE_DIR_CID, su_JS_DIRUTILS_I_PROPS).get(aAppID, su_JS_DIRUTILS_NSIFILE).path; 
  } catch (e) { rv = su_jslibError(e); }

  return rv;
},

getPrefsDir : function () { return this.getPath(su_NS_APP_PREFS_50_DIR); },
getChromeDir : function () { return this.getPath(su_NS_APP_CHROME_DIR); },
getMozHomeDir : function () { return this.getPath(su_NS_APP_USER_PROFILES_ROOT_DIR); },
getMozUserHomeDir : function () { return this.getPath(su_NS_APP_USER_PROFILE_50_DIR); },
getAppRegDir : function () { return this.getPath(su_NS_APP_APPLICATION_REGISTRY_FILE); },
getAppDefaultDir : function () { return this.getPath(su_NS_APP_DEFAULTS_50_DIR); },
getAppDefaultPrefDir : function () { return this.getPath(su_NS_APP_PREF_DEFAULTS_50_DIR); },
getProfileDefaultsLocDir : function () { return this.getPath(su_NS_APP_PROFILE_DEFAULTS_50_DIR); },
getProfileDefaultsDir : function () { return this.getPath(su_NS_APP_PROFILE_DEFAULTS_NLOC_50_DIR); },
getAppResDir : function () { return this.getPath(su_NS_APP_RES_DIR); },
getAppPluginsDir : function () { return this.getPath(su_NS_APP_PLUGINS_DIR); },
getSearchPluginsDir : function ()
{
	var retval;
	
	if (navigator.userAgent.indexOf("Win") != -1)
	{
		retval = this.getPath(su_NS_APP_SEARCH_DIR);
	}
	else if (navigator.userAgent.indexOf("Mac OS X") != -1)
	{
		retval = null;
		// retval = this.getMozHomeDir() + su_SLASH + '..' + su_SLASH + 'searchplugins';
	}
	else if (navigator.userAgent.indexOf("Mac") != -1)
	{
		retval = null;
		// retval = this.getPath(su_NS_APP_SEARCH_DIR);
	}
	else if (navigator.userAgent.indexOf("Linux") != -1)
	{
		retval = null;
		
		// This works on Debian Sarge but not on Fedora Core 3.
		// retval = this.getMozHomeDir() + su_SLASH + '..' + su_SLASH + 'searchplugins';
	}
	else
	{
		retval = this.getPath(su_NS_APP_SEARCH_DIR);
	}
	
	return retval;

},
getPrefsFile : function () { return this.getPath(su_NS_APP_PREFS_50_FILE); },
getUserChromeDir : function () { return this.getPath(su_NS_APP_USER_CHROME_DIR); },
getLocalStore : function () { return this.getPath(su_NS_APP_LOCALSTORE_50_FILE); },
getHistoryFile : function () { return this.getPath(su_NS_APP_HISTORY_50_FILE); },
getPanelsFile : function () { return this.getPath(su_NS_APP_USER_PANELS_50_FILE); },
getMimeTypes : function () { return this.getPath(su_NS_APP_USER_MIMETYPES_50_FILE); },
getBookmarks : function () { return this.getPath(su_NS_APP_BOOKMARKS_50_FILE); },
getSearchFile : function () { return this.getPath(su_NS_APP_SEARCH_50_FILE); },
getUserMailDir : function () { return this.getPath(su_NS_APP_MAIL_50_DIR); },
getUserImapDir : function () { return this.getPath(su_NS_APP_IMAP_MAIL_50_DIR); },
getUserNewsDir : function () { return this.getPath(su_NS_APP_NEWS_50_DIR); },
getMessengerFolderCache : function () { return this.getPath(su_NS_APP_MESSENGER_FOLDER_CACHE_50_DIR); },
getCurProcDir : function () { return this.getPath(su_NS_OS_CURRENT_PROCESS_DIR); },
getHomeDir : function () { return this.getPath(su_NS_OS_HOME_DIR); },
getTmpDir : function () { return this.getPath(su_NS_OS_TEMP_DIR); },
getComponentsDir : function () { return this.getPath(su_NS_XPCOM_COMPONENT_DIR); },

get help() {
  const help =

    "\n\nFunction and Attribute List:\n"    +
    "\n"                                    +
    "    getPrefsDir()\n"                       +
    "    getChromeDir()\n"                      +
    "    getMozHomeDir()\n"                     +
    "    getMozUserHomeDir()\n"                 +
    "    getAppRegDir()\n"                      +
    "    getAppDefaultDir()\n"                  +
    "    getAppDefaultPrefDir()\n"              +
    "    getProfileDefaultsLocDir()\n"          +
    "    getProfileDefaultsDir()\n"             +
    "    getAppResDir()\n"                      +
    "    getAppPluginsDir()\n"                  +
    "    getSearchPluginsDir()\n"               +
    "    getPrefsFile()\n"                      +
    "    getUserChromeDir()\n"                  +
    "    getLocalStore()\n"                     +
    "    getHistoryFile()\n"                    +
    "    getPanelsFile()\n"                     +
    "    getMimeTypes()\n"                      +
    "    getBookmarks()\n"                      +
    "    getSearchFile()\n"                     +
    "    getUserMailDir()\n"                    +
    "    getUserImapDir()\n"                    +
    "    getUserNewsDir()\n"                    +
    "    getMessengerFolderCache()\n\n";

  return help;
}

}; //END CLASS 

su_jslibDebug('*** load: '+su_JS_DIRUTILS_FILE+' OK');

} // END BLOCK su_JS_LIB_LOADED CHECK

else {
    dump("JSLIB library not loaded:\n"                                  +
         " \tTo load use: chrome://jslib/content/jslib.js\n"            +
         " \tThen: su_include(su_jslib_dirutils);\n\n");
}

