/*** -*- Mode: Javascript; tab-width: 2;

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is jslib team code.
The Initial Developer of the Original Code is jslib team.

Portions created by jslib team are
Copyright (C) 2000 jslib team.  All
Rights Reserved.

Original Author: Pete Collins <pete@mozdevgroup.com>
Contributor(s): Martin Kutschker <Martin.T.Kutschker@blackbox.net>

***/

/** 
 * insure su_jslib base is not already loaded
 */
if (typeof(su_JS_LIB_LOADED) == "undefined") 
{
  try {

  const su_JS_LIB_LOADED     = true;

  const su_JS_LIBRARY        = "su_jslib";
  const su_JS_LIB_FILE       = "jslib.js"
  const su_JS_LIB_PATH       = "chrome://jslib/content/";
  const su_JS_LIB_VERSION    = "0.1.245";
  const su_JS_LIB_AUTHORS    = "\tPete Collins       <pete@mozdevgroup.com>\n"   
                          + "\tNeil Deakin        <neil@mozdevgroup.com>\n"   
                          + "\tBrian King         <brian@mozdevgroup.com>\n"   
                          + "\tEric Plaster       <plaster@urbanrage.com>\n"   
                          + "\tMartin.T.Kutschker <Martin.T.Kutschker@blackbox.net>\n";
  const su_JS_LIB_BUILD      = "mozilla 1.3+";
  const su_JS_LIB_ABOUT      = "\tThis is an effort to provide a fully "        
                          + "functional js library\n"                   
                          + "\tfor mozilla package authors to use "    
                          + "in their applications\n";
  const su_JS_LIB_HOME       = "http://jslib.mozdev.org/";

  const su_JS_LIB_ON         = true;
  const su_JS_LIB_OFF        = false;

//  removed as part of StumbleUpon embedding
//  if (typeof(C) == "undefined")
//    var C                 = Components;

  const su_jsC               = Components;
  const su_jslibRes          = su_jsC.results;
  const su_jslibI            = su_jsC.interfaces;

  const su_JS_LIB_OK         = su_jslibRes.NS_OK;
  
  // DEPRICATED
  const su_jslib_results     = su_jslibRes;

  if (typeof(su_JS_LIB_DEBUG) != "boolean")
    var su_JS_LIB_DEBUG      = su_JS_LIB_OFF;

  var su_JS_LIB_DEBUG_ALERT  = su_JS_LIB_OFF;
  var su_JS_LIB_ERROR        = su_JS_LIB_ON;
  var su_JS_LIB_ERROR_ALERT  = su_JS_LIB_OFF;

  const su_JS_LIB_HELP       = "\n\nWelcome to su_jslib version "+su_JS_LIB_VERSION+"\n\n" 
                          + "Global Constants:\n\n"                               
                          + "su_JS_LIBRARY     \n\t"+su_JS_LIBRARY     +"\n"
                          + "su_JS_LIB_FILE    \n\t"+su_JS_LIB_FILE    +"\n"                 
                          + "su_JS_LIB_PATH    \n\t"+su_JS_LIB_PATH    +"\n"
                          + "su_JS_LIB_VERSION \n\t"+su_JS_LIB_VERSION +"\n"
                          + "su_JS_LIB_AUTHORS \n"  +su_JS_LIB_AUTHORS
                          + "su_JS_LIB_BUILD   \n\t"+su_JS_LIB_BUILD   +"\n" 
                          + "su_JS_LIB_ABOUT   \n"  +su_JS_LIB_ABOUT
                          + "su_JS_LIB_HOME    \n\t"+su_JS_LIB_HOME    +"\n\n"
                          + "Global Variables:\n\n"            
                          + "  su_JS_LIB_DEBUG\n  su_JS_LIB_ERROR\n\n";


  function su_jslibGetService (aURL, aInterface) 
  {
    var rv;
    try {
      // determine how 'aInterface' is passed and handle accordingly
      switch (typeof(aInterface))
      {
        case "object":
          rv = su_jsC.classes[aURL].getService(aInterface);
          break;

        case "string":
          rv = su_jsC.classes[aURL].getService(su_jsC.interfaces[aInterface]);
          break;
     
        default:
          rv = su_jsC.classes[aURL].getService();
          break;
      }
    } catch (e) { rv = -1; }

    return rv;
  }

  function su_jslibCreateInstance (aURL, aInterface) 
  {
    var rv;
    try {
      rv = su_jsC.classes[aURL].createInstance(su_jsC.interfaces[aInterface]);
    } catch (e) {
      rv = -1;
    }
    return rv;
  }
 
  function su_jslibGetInterface (aInterface) 
  {
    var rv;
    try {
      rv = su_jsC.interfaces[aInterface];
    } catch (e) {
      rv = e;
    }
    return rv;
  }
 
  function su_jslibQI (aObj, aInterface)
  {
    try {
      return aObj.QueryInterface(su_jslibI[aInterface]);
    } catch (e) {
      return e;
    }
  }
 
  /****************************************************************
  * void su_include(aScriptPath)                                     *
  * aScriptPath is an argument of string lib chrome path          *
  * returns NS_OK on success, 1 if file is already loaded and     *
  * - errorno or throws exception on failure                      *
  *   eg:                                                         * 
  *       var path='chrome://su_jslib/content/io/file.js';           *
  *       su_include(path);                                          *
  *  Or:                                                          *
  *       su_include(su_jslib_file);                                    *
  *                                                               *
  *   outputs: void(null)                                         *
  ****************************************************************/

  function su_include (aScriptPath) 
  {
    if (!aScriptPath) {
      dump("su_include: Missing file path argument\n");
      throw - su_jslibRes.NS_ERROR_XPC_NOT_ENOUGH_ARGS;
    }

    if (aScriptPath == su_JS_LIB_PATH+su_JS_LIB_FILE) {
      dump("su_include: "+aScriptPath+" is already loaded!\n");
      throw - su_jslibRes.NS_ERROR_INVALID_ARG;
    }

    var start   = aScriptPath.lastIndexOf('/') + 1;
    var end     = aScriptPath.lastIndexOf('.');
    var slice   = aScriptPath.length - end;
    var loadID  = aScriptPath.substring(start, (aScriptPath.length - slice));

    if (typeof(this['su_JS_'+loadID.toUpperCase()+'_LOADED']) == "boolean")
      return su_jslibRes.NS_OK;

    var rv;
    try {
      if (su_jslibNeedsPrivs())
        netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
      su_jslibGetService("@mozilla.org/moz/jssubscript-loader;1", 
                      "mozIJSSubScriptLoader").loadSubScript(aScriptPath);
      rv = su_jslibRes.NS_OK;
      dump("su_include: "+aScriptPath+"\n");
    } catch (e) {
      const msg = aScriptPath+" is not a valid path or is already loaded\n";
      dump(e+"\n");
      dump("su_include: "+msg+"\n");
      rv = - su_jslibRes.NS_ERROR_INVALID_ARG;
    }
    return rv;
  }

  function su_jslibNeedsPrivs ()
  {
    var rv;
    if (typeof(this.location) == "object") {
      var proto = this.location.protocol;
      rv = (proto == "file:")
    }
    return rv;
  }

  // su_include debug methods
  const su_jslib_debug = su_JS_LIB_PATH+'debug/debug.js';
  su_include(su_jslib_debug);

  function su_jslibUninstall (aPackage, aCallback)
  {
    if (!aPackage)
      throw - su_jslibRes.NS_ERROR_INVALID_ARG;

    su_include (su_jslib_window);
    var win = new CommonWindow(null, 400, 400);
    win.position = su_JS_MIDDLE_CENTER;
    win.openUninstallWindow(aPackage, aCallback);
  }

  /*********** Launch JSLIB Splash ***************/
  function su_jslibLaunchSplash ()
  {
    su_include (su_jslib_window);
    var win = new CommonWindow("chrome://su_jslib/content/splash.xul", 400, 220);
    win.position = su_JS_MIDDLE_CENTER;
    win.openSplash();
  }

  function su_jslibLaunchConsole ()
  {
    su_include (su_jslib_window);
    var win = new CommonWindow("javascript:", 400, 220);
    win.position = su_JS_MIDDLE_CENTER;
    win.open();
  }

  // DEPRICATED
  function su_jslib_turnDumpOn () { su_jslibTurnDumpOn(); }

  function su_jslibTurnDumpOn () 
  {
    su_include (su_jslib_prefs);
    // turn on dump
    var pref = new su_Prefs();
    const prefStr = "browser.dom.window.dump.enabled"

    // turn dump on if not enabled
    if (!pref.getBool(prefStr)) {
      pref.setBool(prefStr, true);
      pref.save();
    } 
    return;
  }

  // DEPRICATED
  function su_jslib_turnDumpOff () { su_jslibTurnDumpOff(); }

  function su_jslibTurnDumpOff () 
  {
    su_include (su_jslib_prefs);
    // turn off dump
    var pref = new su_Prefs();
    const prefStr = "browser.dom.window.dump.enabled"

    // turn dump off if enabled
    if (pref.getBool(prefStr)) {
      pref.setBool(prefStr, false);
      pref.save();
    } 
    return;
  }
 
  function su_jslibTurnStrictOn () 
  {
    su_include (su_jslib_prefs);
    // turn on dump
    var pref = new su_Prefs();
    const prefStr = "javascript.options.strict";

    // turn dump on if not enabled
    if (!pref.getBool(prefStr)) {
      pref.setBool(prefStr, true);
      pref.save();
    } 
    return;
  }

  function su_jslibTurnStrictOff () 
  {
    su_include (su_jslib_prefs);
    // turn off dump
    var pref = new su_Prefs();
    const prefStr = "javascript.options.strict";

    // turn dump off if enabled
    if (pref.getBool(prefStr)) {
      pref.setBool(prefStr, false);
      pref.save();
    } 
    return;
  }

  const su_jslib_modules = su_JS_LIB_PATH+'modules.js';
  su_include (su_jslib_modules);

  } catch (e) {}

} // end su_jslib load test

