/*** -*- Mode: Javascript; tab-width: 2;
  
The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is jslib team code.  
The Initial Developer of the Original Code is jslib team.

Portions created by jslib team are
Copyright (C) 2000 jslib team.  All
Rights Reserved.
  
Original Author: Pete Collins <pete@mozdev.org>
Contributor(s):
    
***/



//************** BEGIN UTILITY FUNCTIONS *****************/

// These used to be in stumbleuponOverlay.js, but they've been 
// moved here so that the preferenceDialog has access to them 
// even when it's opened from the Extensions window of Firefox 
// and no Firefox browser window is open.
//
// NOTE:  Objects su_pr and stumbleid must be defined on the 
// window object of the calling function prior to calling these.



// Preference wrappers
function su_pref_setstring(key, value)
{
	su_pr.setChar("stumble." + stumbleid + "." + key, value);
}

function su_pref_getstring(key)
{
	return su_pr.getChar("stumble." + stumbleid + "." + key, "");
}

function su_pref_setstring_absolute(key, value)
{
	su_pr.setChar("stumble." + key, value);
}

function su_pref_getstring_absolute(key)
{
	return su_pr.getChar("stumble." + key, "");
}

function su_pref_getorsetstring_absolute(key, value)
{
	return su_pr.getOrSetChar("stumble." + key, value);
}

function su_pref_setint(key, value)
{
	su_pr.setInt("stumble." + stumbleid + "." + key, value);
}

function su_pref_getint(key)
{
	return su_pr.getInt("stumble." + stumbleid + "." + key, "");
}

function su_pref_getorsetint(key, value)
{
	return su_pr.getOrSetInt("stumble." + stumbleid + "." + key, value);
}

function su_pref_setint_absolute(key, value)
{
	su_pr.setInt("stumble." + key, value);
}

function su_pref_getint_absolute(key)
{
	return su_pr.getInt("stumble." + key, "");
}

function su_pref_setbool(key, value)
{
	su_pr.setBool("stumble." + stumbleid + "." + key, value);
}

function su_pref_getbool(key)
{
	return su_pr.getBool("stumble." + stumbleid + "." + key, false);
}

function su_pref_getorsetbool(key, value)
{
	return su_pr.getOrSetBool("stumble." + stumbleid + "." + key, value);
}

function su_pref_getorsetstring(key, value)
{
	return su_pr.getOrSetChar("stumble." + stumbleid + "." + key, value);
}

//************** END UTILITY FUNCTIONS *****************/



if (typeof(su_JS_LIB_LOADED)=='boolean') 
{

  const su_JS_PREFS_LOADED   = true;
  const su_JS_PREFS_FILE     = 'prefs.js';
  const su_JS_PREFS_CID      = "@mozilla.org/preferences;1";
  const su_JS_PREFS_I_PREF   = "nsIPref";

  /*********** PREFS *******************/
  function su_Prefs () 
  {
    try {
      // create instance of prefs xpcom object
      this.prefInst = su_jslibCreateInstance(su_JS_PREFS_CID, su_JS_PREFS_I_PREF);
      this.prefInst = su_jslibQI(this.prefInst, "nsIPrefBranch");
    } catch (e) { su_jslibError(e); }
  }

  su_Prefs.prototype = 
  {
    prefInst: null,

  /*********** GET Type ****************/
  // pref type
  getType : function (aPrefString) 
  {
    var rv;
    try {
      rv = this.prefInst.GetPrefType(aPrefString)
    } catch (e) {
      su_jslibError(e);
      rv=undefined;
    }
    return rv;
  },
  /*********** GET Type ****************/

  /*********** SET PREF ****************/
  // bool
  setBool : function (aPrefString, aInBool) 
  {
    if (!aPrefString)
      return false;

    var rv;
    try {
      this.prefInst.SetBoolPref(aPrefString, aInBool)
      rv = su_jslib_results.NS_OK;

    } catch(e) {
			su_jslibError(e);
    	rv=-su_jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** SET PREF ****************/

  /*********** GET PREF ****************/
  // bool
  getBool : function (aPrefString) 
  {
    if (!aPrefString)
      return false;

    var rv;
    try {
      rv = this.prefInst.GetBoolPref(aPrefString)
    } catch(e) {
    	rv=false;
    }
    return rv;
  },
 getOrSetBool : function (aPrefString, aDefault) {
   if (!aPrefString)
     throw "su_jslib:getOrSetBool not enough args provided\n";
 
   var doset = false;
   var rv;
   try {
     rv = this.prefInst.GetBoolPref(aPrefString)
   } catch(e) {
     	rv=aDefault;
 			doset = true;
   }
   if (typeof(rv) != 'boolean' || doset) {
 		rv = aDefault;
 		try {
 			this.prefInst.SetBoolPref(aPrefString, rv);
 		}	catch (x) {
     	throw "su_jslib:getOrSetBool could not set boolean preference\n";
 		} 
 	}
 
   return rv;
 },
  /*********** GET PREF ****************/

  /*********** SET CHAR PREF ***********/
  // nsresult
  setChar : function (aPrefName, aPrefString) 
  {
    if (!aPrefName && !aPrefString)
      throw "su_jslib:setChar not enough args provided\n";

    var rv;
    try {
      this.prefInst.SetCharPref(aPrefName, aPrefString)
      rv = su_jslib_results.NS_OK;

    } catch(e) {
			su_jslibError(e);
    	rv=-su_jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** SET CHAR PREF ***********/

  /*********** GET CHAR PREF ***********/
  // string
  getChar : function (aPrefName) 
  {
    if (!aPrefName)
      throw "su_jslib:getChar not enough args provided\n";

    var rv = "";
    try {
      rv = this.prefInst.GetCharPref(aPrefName)
    } catch(e) {
    	rv="";
    }
    return rv;
  },
 getOrSetChar : function (aPrefString, aDefault) {
   if (!aPrefString)
     throw "su_jslib:getOrSetChar not enough args provided\n";
 
   var doset = false;
   var rv;
   try {
     rv = this.prefInst.GetCharPref(aPrefString)
   } catch(e) {
     	rv=aDefault;
 			doset = true;
   }
   if (typeof(rv) != 'string' || doset) {
 		rv = aDefault;
 		try {
 			this.prefInst.SetCharPref(aPrefString, rv);
 		}	catch (x) {
     	throw "su_jslib:getOrSetChar could not set string preference\n";
 		} 
 	}
 
   return rv;
 },
  /*********** SET CHAR PREF ***********/

  /*********** SET INT PREF ***********/
  // nsresult
  setInt : function (aPrefName, aInt) 
  {
    if (!aPrefName && !aInt)
      throw "su_jslib:setInt not enough args provided\n";

    var rv;
    try {
      this.prefInst.SetIntPref(aPrefName, aInt)
// replaced as part of StumbleUpon embedding
//      rv = C.results.NS_OK;
      rv = su_jsC.results.NS_OK;

    } catch(e) {
			su_jslibError(e);
    	rv=-su_jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** SET INT PREF ***********/

  /*********** GET INT PREF ***********/
  // int
  getInt : function (aPrefName) 
  {
    if (!aPrefName)
      throw "su_jslib:getInt not enough args provided\n";

    var rv;
    try {
      rv = this.prefInst.GetIntPref(aPrefName)
    } catch(e) {
    	rv=-su_jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
 /*********** GET OR SET INT PREF ***********/
 // int
 getOrSetInt : function (aPrefName, aDefault) {
 
   if (!aPrefName)
     throw "su_jslib:getOrSetInt not enough args provided\n";
 
   var doset = false;
   var rv;
   try {
     rv = this.prefInst.GetIntPref(aPrefName)
   } catch(e) {
 	// su_jslibError(e, "Failed to get int pref", "NS_ERROR_FAILURE", su_JS_PREFS_FILE);
     	rv=aDefault;
 	doset = true;
   }
 
 
 	if (typeof(rv) != 'number' || doset) 
 	{
 		rv = aDefault;
 		try 
 		{
 			this.prefInst.SetIntPref(aPrefName, rv)
 		} catch (x) {
 			throw "su_jslib:getOrSetInt could not set int preference\n";
 		}
 	}
 
   return rv;
 },

  /*********** SET INT PREF ***********/

  /*********** RESET PREF *************/
  // nsresult
  reset : function () 
  {
    var rv;
    try {
      this.prefInst.Resetsu_Prefs();
      rv = su_jslib_results.NS_OK;
    } catch(e) {
			su_jslibError(e);
    	rv=-su_jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** RESET PREF *************/

  /*********** RESET USER PREF ********/
  // nsresult
  resetUser : function () 
  {
    var rv;
    try {
      this.prefInst.ResetUsersu_Prefs();
      rv = su_jslib_results.NS_OK;
    } catch(e) {
			su_jslibError(e);
    	rv=-su_jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** RESET USER PREF ********/

  /*********** SAVE PREF **************/
  // nsresult
  save : function (aFile) 
  {
    var file;
    switch (typeof(aFile)) 
    {
      case "object":
        // check object is an nsIFile object
        if (typeof(aFile.path) == "string")
        file = aFile;
        break;

      case "string":
        su_include (su_jslib_file);
        // path is a string, make it into an nsIFile
        file = (new su_File(aFile)).nsIFile;
        break;
    }

    var rv;
    try {
      this.prefInst.savePrefFile(file);
      rv = su_jslib_results.NS_OK;
    } catch(e) {
			su_jslibError(e);
    	rv=-su_jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** SAVE PREF **************/

  /*********** CLEAR USER PREF ********/
  // nsresult
  clear : function (aPrefString) 
  {
    var rv;
    try {
      this.prefInst.ClearUserPref(aPrefString);
      rv = su_jslib_results.NS_OK;
    } catch(e) {
			su_jslibError(e);
    	rv=-su_jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** CLEAR USER PREF ********/

  /*********** GET NSIPREF ********/
  get nsIPref () { return this.prefInst; }
  /*********** GET NSIPREF ********/

  } // END CLASS
  /*********** PREFS *******************/

  su_jslibDebug('*** load: '+su_JS_PREFS_FILE+' OK');

} // END BLOCK su_JS_LOADED CHECK
    
// If su_jslib base library is not loaded, dump this error.
else
{  
    dump("su_JS_PREFS library not loaded:\n"                           +
         " \tTo load use: chrome://su_jslib/content/su_jslib.js\n"        +
         " \tThen: su_include (su_jslib_prefs);\n\n");
}
